#ifdef USE_DDRAW
#ifndef lint
static char rcsid[] =
"@(#) $Header: /cvs/fl/ag-media/vic/video/assistor-intel-jpeg-new.cpp,v 1.1.1.1 2003/12/09 20:52:00 judson Exp $ (LBL)";
#endif

#include "vw.h"
#include <stdlib.h>
#include <sys/types.h>

#include "inet.h"
#include "vic_tcl.h"
#include "rtp.h"
#include "decoder.h"
#include "renderer.h"
#include "iohandler.h"
#include "assistor-intel-jpeg.h"
#include <tkWin.h>

extern char *ddrawErrorString(HRESULT rc);


//
// Matcher object for creating the initial instance 
// of the assistor.
//

static IntelJpegAssistorMatcher assistor_intel;

int __STDCALL errorCallback(IPLStatus status, const char * funcname,
		  const char * context, const char * file, int line)
{
    debug_msg("IPL error: %s: %s %s at %s:%d\n",
	iplErrorStr(status), funcname, context, file, line);
    return 0;
}

IntelJpegAssistorMatcher::IntelJpegAssistorMatcher() : Matcher("assistor") 
{
    iplRedirectError(errorCallback);
    debug_msg("create intel jpeg assistor\n");
}

TclObject* IntelJpegAssistorMatcher::match(const char* id) 
{
    debug_msg("match assistor to %s\n", id);
    if (strcmp(id, "jpeg/422") == 0) 
    {
	IntelJpegAssistor* p = new IntelJpegAssistor();
	return (p);
    }
    return (0);
}


IntelJpegAssistor::IntelJpegAssistor()
	: Renderer(FT_JPEG), decoder_(0), renderer_(0)
{
    debug_msg("create jpeg assistor\n");
    inw_ = inh_ = inq_ = 0;
}

IntelJpegAssistor::~IntelJpegAssistor()
{
    debug_msg("Destroy assistor\n");

    destroyComponents();
}

int IntelJpegAssistor::command(int argc, const char*const* argv)
{
    if (argc == 3) 
    {
	if (strcmp(argv[1], "window") == 0) 
	{
	    window_ = VideoWindow::lookup(argv[2]);
	    
	    debug_msg("got window %s %x size=%dx%d\n", argv[2], window_,
		window_->width(), window_->height());

	    rw_ = window_->width();
	    rh_ = window_->height();
	    
	    setupComponents();

	    return (TCL_OK);
	}
    }
    return (Renderer::command(argc, argv));
}

void IntelJpegAssistor::setupComponents()
{
    if (inw_ == 0)
	return;

    int bpp = window_->bpp();
#define TRY_OVERLAY
#ifdef TRY_OVERLAY
    //
    // Try to create an overlay
    //

    JRenderer_24_Overlay *overlayRenderer = new JRenderer_24_Overlay(this);
    if (overlayRenderer->create())
    {
	debug_msg("Created overlay\n");

	decoder_ = new JDecoder_YCbCr(this, overlayRenderer);
	renderer_ = overlayRenderer;
	return;
    }
    else
    {
	delete overlayRenderer;
    }
#endif

    if (bpp == 24)
    {

	if (inw_ == rw_ && inh_ == rh_)
	{
	    debug_msg("configure 24bit same\n");
	    renderer_ = new JRenderer_24_DDrawVideoImage(this);
	    decoder_ = new JDecoder_24_RGB_same(this, renderer_);
	}
	else if (inw_ <= rw_ && inh_ <= rh_)
	{
	    debug_msg("configure 24bit zoom\n");
	    renderer_ = new JRenderer_24_DDrawVideoImage(this);
	    decoder_ = new JDecoder_24_RGB_zoom(this, renderer_);
	}
	else if (inw_ >= rw_ && inh_ >= rh_)
	{
	    debug_msg("configure 24bit decimate\n");
	    renderer_ = new JRenderer_24_DDrawVideoImage(this);
	    decoder_ = new JDecoder_24_RGB_decimate(this, renderer_);
	}
    
    }
}

int IntelJpegAssistor::consume(const VideoFrame* vf)
{
    JpegFrame *jf = (JpegFrame *) vf;
    
    if (jf->width_ != inw_ || jf->height_ != inh_)
	resetSize(jf->width_, jf->height_);
    
    if (jf->q_ != inq_)
	resetQ(jf->q_);

    if (decoder_ != 0)
    {
	int rc;
	rc = decoder_->consume(jf);
	return rc;
    }
    else
    {
	return 0;
    }
}

void IntelJpegAssistor::resetQ(int inq)
{
    
    inq_ = inq;

    if (decoder_ != 0)
    {
	decoder_->resetQ(inq_);
    }
}


//
// resetSize is invoked when the size of the incoming stream
// changes. For now, we'll tear down the decoder and renderer
// and rebuild them.
//
void IntelJpegAssistor::resetSize(int inw, int inh)
{
    
    debug_msg("jpeg resetSize w=%d h=%d\n", inw, inh);
    /*
    * Make sure all outstanding putimage's finish.
    * Syncing with the first window, syncs the server,
    * and hence all of them.
    */
    window_->sync();
    
    Module::size(inw, inh);
    inw_ = inw;
    inh_ = inh;

    destroyComponents();
    setupComponents();
}

void IntelJpegAssistor::destroyComponents()
{
    if (decoder_ != 0)
    {
	delete decoder_;
	decoder_ = 0;
    }
    if (renderer_ != 0)
    {
	delete renderer_;
	renderer_ = 0;
    }
}

//
// Decoder classes
//
IntelJpegAssistor::JDecoder::JDecoder(IntelJpegAssistor *assistor, 
				      JRenderer *renderer)
				      : assistor_(assistor),renderer_(renderer)
{
    ZeroMemory(&prop_, sizeof(prop_));
}

IntelJpegAssistor::JDecoder::~JDecoder()
{
}

//
// 24-bit jpeg decode, RGB, render to same-sized window. 
//

IntelJpegAssistor::JDecoder_24_RGB_same::JDecoder_24_RGB_same(IntelJpegAssistor *assistor, 
						      JRenderer *renderer)
						      : JDecoder(assistor, renderer)
{
    IJLERR err = ijlInit(&prop_);
    
    if (err < 0)
    {
	debug_msg("ijlInit failed: %s\n", ijlErrorStr(err));
    }
    prop_.DIBChannels = 3;
    prop_.DIBColor = IJL_BGR;
    prop_.JPGSubsampling = IJL_422;

    prop_.DIBWidth = assistor_->inw_;
    prop_.DIBHeight = assistor_->inh_;

 }

IntelJpegAssistor::JDecoder_24_RGB_same::~JDecoder_24_RGB_same()
{
    ijlFree(&prop_);
}

int IntelJpegAssistor::JDecoder_24_RGB_same::consume(const JpegFrame *jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;

    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;

    IplImage *rImage = renderer_->getImage();
    prop_.DIBBytes = (unsigned char *) rImage->imageData;

    err = ijlRead(&prop_, readType);

    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
    }
    else
    {
	renderer_->render();
    }

    return 0;
}

//
// 24-bit jpeg decode, RGB, render to larger window. 
//

IntelJpegAssistor::JDecoder_24_RGB_zoom::JDecoder_24_RGB_zoom(IntelJpegAssistor *assistor, 
						      JRenderer *renderer)
						      : JDecoder(assistor, renderer)
{
    IJLERR err = ijlInit(&prop_);

    
    if (err < 0)
    {
	debug_msg("ijlInit failed: %s\n", ijlErrorStr(err));
    }
    prop_.DIBChannels = 3;
    prop_.DIBColor = IJL_BGR;
    prop_.JPGSubsampling = IJL_422;
    
    int inw_ = assistor_->inw_;
    int inh_ = assistor_->inh_;

    
    work_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	inw_, inh_, 0, 0, 0, 0);
    IPL_ERRCHK("reset_24", "iplCreateImageHeader");
    
    if (work_ == 0)
    {
	debug_msg("create image header failed\n");
    }
    
    iplAllocateImage(work_, 0, 0);
    IPL_ERRCHK("reset_24", "iplAllocateImage");
    
    prop_.DIBBytes = (unsigned char*) work_->imageData;
    prop_.DIBWidth = inw_;
    prop_.DIBHeight = inh_;

 }

IntelJpegAssistor::JDecoder_24_RGB_zoom::~JDecoder_24_RGB_zoom()
{
    ijlFree(&prop_);
    iplDeallocateHeader(work_);
}

int IntelJpegAssistor::JDecoder_24_RGB_zoom::consume(const JpegFrame *jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;

    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;

    err = ijlRead(&prop_, readType);

    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
    }
    else
    {
	iplZoom(work_, renderer_->getImage(),
	    assistor_->rw_, assistor_->inw_,
	    assistor_->rh_, assistor_->inh_,
	    IPL_INTER_LINEAR);
	renderer_->render();
    }

    return 0;
}


//
// 24-bit jpeg decode, RGB, render to smaller window. 
//

IntelJpegAssistor::JDecoder_24_RGB_decimate::JDecoder_24_RGB_decimate(IntelJpegAssistor *assistor, 
						      JRenderer *renderer)
						      : JDecoder(assistor, renderer)
{
    IJLERR err = ijlInit(&prop_);

    
    if (err < 0)
    {
	debug_msg("ijlInit failed: %s\n", ijlErrorStr(err));
    }
    prop_.DIBChannels = 3;
    prop_.DIBColor = IJL_BGR;
    prop_.JPGSubsampling = IJL_422;
    
    int inw_ = assistor_->inw_;
    int inh_ = assistor_->inh_;

    
    work_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	inw_, inh_, 0, 0, 0, 0);
    IPL_ERRCHK("reset_24", "iplCreateImageHeader");
    
    if (work_ == 0)
    {
	debug_msg("create image header failed\n");
    }
    
    iplAllocateImage(work_, 0, 0);
    IPL_ERRCHK("reset_24", "iplAllocateImage");
    
    prop_.DIBBytes = (unsigned char*) work_->imageData;
    prop_.DIBWidth = inw_;
    prop_.DIBHeight = inh_;

 }

IntelJpegAssistor::JDecoder_24_RGB_decimate::~JDecoder_24_RGB_decimate()
{
    ijlFree(&prop_);
    iplDeallocateHeader(work_);
}

int IntelJpegAssistor::JDecoder_24_RGB_decimate::consume(const JpegFrame *jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;

    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;

    err = ijlRead(&prop_, readType);

    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
    }
    else
    {
	iplDecimate(work_, renderer_->getImage(),
	    assistor_->rw_, assistor_->inw_,
	    assistor_->rh_, assistor_->inh_,
	    IPL_INTER_LINEAR);
	renderer_->render();
    }

    return 0;
}

//
// Renderer abstract superclass
//

IntelJpegAssistor::JRenderer::JRenderer(IntelJpegAssistor *a)
: assistor_(a) 
{
}

IntelJpegAssistor::JRenderer::~JRenderer()
{
}

//
// DDrawVideoImage-based renderer
//

IntelJpegAssistor::JRenderer_24_DDrawVideoImage::
JRenderer_24_DDrawVideoImage(IntelJpegAssistor *a)
: JRenderer(a) 
{
    videoImage_ = new DDrawVideoImage(assistor_->window_->tkwin(), 
	assistor_->rw_, assistor_->rh_);

    debug_msg("create, image size is %dx%d\n", assistor_->rw_, assistor_->rh_);
    image_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_4BYTES, 
	assistor_->rw_, assistor_->rh_, 0, 0, 0, 0);
    
    IPL_ERRCHK("reset_24", "iplCreateImageHeader");
    
    if (image_ == 0)
    {
	debug_msg("create image header failed\n");
    }
    
    image_->imageData = (char *) videoImage_->pixbuf();
    image_->imageSize = assistor_->rw_ * assistor_->rh_ * 3;
}

IntelJpegAssistor::JRenderer_24_DDrawVideoImage::~JRenderer_24_DDrawVideoImage()
{
    image_->imageData = 0;
    iplDeallocateHeader(image_);
    delete videoImage_;
}

IplImage *IntelJpegAssistor::JRenderer_24_DDrawVideoImage::getImage()
{
    return image_;
}

void IntelJpegAssistor::JRenderer_24_DDrawVideoImage::render()
{
    assistor_->window_->render(videoImage_, 0, assistor_->rh_, 0, assistor_->rw_);

}
IntelJpegAssistor::JDecoder_YCbCr::JDecoder_YCbCr(IntelJpegAssistor *assistor, 
						  JRenderer_24_Overlay *renderer)
    : JDecoder(assistor, renderer), overlayRenderer_(renderer)
{
    IJLERR err = ijlInit(&prop_);
    
    if (err < 0)
    {
	debug_msg("ijlInit failed: %s\n", ijlErrorStr(err));
    }

    prop_.JPGColor = IJL_YCBCR;
    prop_.JPGSubsampling = IJL_422;

    prop_.DIBChannels = 2;
    prop_.DIBColor = IJL_YCBCR;

    prop_.DIBWidth = assistor_->inw_;
    prop_.DIBHeight = assistor_->inh_;
}

IntelJpegAssistor::JDecoder_YCbCr::~JDecoder_YCbCr()
{
    ijlFree(&prop_);
}

int IntelJpegAssistor::JDecoder_YCbCr::consume(const JpegFrame *jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;

    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;

    void *image;
    int pitch;

    if (overlayRenderer_->lockBuffer(&image, &pitch))
    {
	prop_.DIBBytes = (unsigned char *) image;

	err = ijlRead(&prop_, readType);

	if (err < 0)
	{
	    debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
	}

	overlayRenderer_->unlockBuffer();
    }

    return 0;
}


//
// Overlay-based renderer
//

IntelJpegAssistor::JRenderer_24_Overlay::
JRenderer_24_Overlay(IntelJpegAssistor *a)
    : JRenderer(a) , overlay_(0)
{
}

//
// We do all the work in the create() method so that we can
// see if an overlay is available.
//

int IntelJpegAssistor::JRenderer_24_Overlay::create()
{
    //
    // Create a DDrawVideoImage, since that object has
    // most of the infrastructure in place to work 
    // with DDraw.
    //

    videoImage_ = new DDrawVideoImage(assistor_->window_->tkwin(), 
	assistor_->rw_, assistor_->rh_);
    videoImage_->checkRecalculate(Tk_WindowId(assistor_->window_->tkwin()));


    //
    // See if we have an overlay available on the monitor .
    //

    int maxOverlays, visibleOverlays;
    DDrawImageMonitor *mon = videoImage_->currentMonitor();

    if (!mon->supportsOverlay(maxOverlays, visibleOverlays))
    {
	debug_msg("Monitor does not support overlays\n");
	return 0;
    }
    else if (visibleOverlays >= maxOverlays)
    {
	debug_msg("No available overlays (max=%d visible=%d)\n",
	    maxOverlays, visibleOverlays);
	return 0;
    }

    debug_msg("Monitor supports overlay max=%d vis=%d\n",
	maxOverlays, visibleOverlays);

    //
    // Snag the appropriate info from the monitor so we can
    // try to create the overlay.
    //

    ddraw_ = mon->monitor()->getDirectDraw();
    LPDIRECTDRAWSURFACE7 surface = mon->monitor()->primarySurface_;

    //
    // Create the overlay. 
    //
    
    HRESULT rc;

    computeRestrictions();

    DDPIXELFORMAT yuy2 = {
	sizeof(DDPIXELFORMAT),
	DDPF_FOURCC,
	MAKEFOURCC('Y','U','Y','2'),
	0,0,0,0,0
    };

    DDSURFACEDESC2 ocaps;

    ZeroMemory(&ocaps, sizeof(ocaps));
    ocaps.dwSize = sizeof(ocaps);

    ocaps.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;

    ocaps.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY;

    ocaps.dwWidth = assistor_->inw_;
    ocaps.dwHeight = assistor_->inh_;
    
    ocaps.ddpfPixelFormat = yuy2;

    rc = ddraw_->CreateSurface(&ocaps, &overlay_, 0);
    if (FAILED(rc))
    {
	debug_msg("Overlay creation failed\n");
	return 0;
    }

    debug_msg("Created overlay!\n");

    windowMoved();

    return 1;
}


void IntelJpegAssistor::JRenderer_24_Overlay::windowMoved()
{
    RECT srcRect, destRect;
    HRESULT rc;

    DDrawImageMonitor *mon = videoImage_->currentMonitor();
    LPDIRECTDRAWSURFACE7 surface = mon->monitor()->primarySurface_;

    mon->getDisplayRect(Tk_GetHWND(Tk_WindowId(assistor_->window_->tkwin())), 
	0, 0, assistor_->rw_, assistor_->rh_, destRect);

    SetRect(&srcRect, 0, 0, assistor_->inw_, assistor_->inh_);

    enforceAlignment(&srcRect, &destRect);

    rc = overlay_->UpdateOverlay(0, surface, &destRect, DDOVER_SHOW, 0);
    if (FAILED(rc))
    {
	debug_msg("Overlay failed: %s\n", ddrawErrorString(rc));
	return;
    }
}

IntelJpegAssistor::JRenderer_24_Overlay::~JRenderer_24_Overlay()
{
    if (overlay_ != 0)
	overlay_->Release();
}

int IntelJpegAssistor::JRenderer_24_Overlay::lockBuffer(void **ptr,
							int *pitch)
{
    DDSURFACEDESC2 ddsd;
    HRESULT hRet;

    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);

    hRet = overlay_->Lock( NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL);
    if (FAILED(hRet))
    {
	debug_msg("lock failed: %s\n", ddrawErrorString(hRet));
	return 0;
    }
    *ptr = ddsd.lpSurface;
    *pitch = ddsd.lPitch;
    return 1;
}

void IntelJpegAssistor::JRenderer_24_Overlay::unlockBuffer()
{
    overlay_->Unlock(NULL);
}
			

void IntelJpegAssistor::JRenderer_24_Overlay::computeRestrictions()
{
    HRESULT rc;
    DDCAPS caps;
    ZeroMemory(&caps, sizeof(caps));
    caps.dwSize = sizeof(caps);

    rc = ddraw_->GetCaps(&caps, NULL);
    
    if (rc != DD_OK)
    {
	debug_msg("getcaps failed with %s\n", ddrawErrorString(rc));
	return;
    }

    if (caps.dwCaps & DDCAPS_OVERLAYSTRETCH)
	stretch_ = (caps.dwMinOverlayStretch > 1000) ? caps.dwMinOverlayStretch  : 1000;
    else
	stretch_ = 1000;
    srcAlign_ = (caps.dwCaps & DDCAPS_ALIGNSIZESRC) ? caps.dwAlignSizeSrc : 0;
    destAlign_ = (caps.dwCaps & DDCAPS_ALIGNSIZEDEST) ? caps.dwAlignSizeDest : 0;

}

void IntelJpegAssistor::JRenderer_24_Overlay::enforceAlignment(RECT *src, RECT *dest)
{
    return;
    if (srcAlign_)
	dest->right -= dest->right % srcAlign_;

    dest->right = (src->right * stretch_ + 999) / 1000;
    dest->bottom = src->bottom * stretch_ / 1000;
}
#endif