
#ifndef lint
static char rcsid[] =
"@(#) $Header: /cvs/fl/ag-media/vic/video/assistor-intel-jpeg.cpp,v 1.1.1.1 2003/12/09 20:52:00 judson Exp $ (LBL)";
#endif

#include "vw.h"
#include <stdlib.h>
#include <sys/types.h>

#include "inet.h"
#include "vic_tcl.h"
#include "rtp.h"
#include "decoder.h"
#include "renderer.h"
#include "iohandler.h"

#include <ijl.h>
#include <ipl.h>

class IntelJpegAssistor : public Renderer {
public:
    IntelJpegAssistor();
    ~IntelJpegAssistor();
    int command(int argc, const char*const* argv);
    
protected:
    virtual int consume(const VideoFrame* vf) ;
    void reset(int inq, int inw, int inh);
    int imagesize_;
    VideoWindow* window_;
    int rw_;		/* width of rendered image */
    int rh_;		/* height of rendered image */
    int inq_;		/* input quantization */
    int inw_;		/* input width */
    int inh_;		/* input height */
    
    DDrawVideoImage *videoImage_;
    
    JPEG_CORE_PROPERTIES prop_;
    
    void init_common();
    void init_24();
    void init_32();
    void init_32_swapped();
    void init_window_24();
    void init_window_32();
    
    
    int consume_24_remap(const JpegFrame* jf) ;
    int consume_24_remap_direct(const JpegFrame* jf) ;
    int consume_24_exact(const JpegFrame* jf) ;
    int consume_32_direct(const JpegFrame* jf) ;
    int consume_32_via_render(const JpegFrame* jf) ;

    void reset_24();
    void reset_32();
    
    typedef int (IntelJpegAssistor::*ConsumeMethod)(const JpegFrame *);
    ConsumeMethod method_;
    
    typedef void (IntelJpegAssistor::*ResetMethod)();
    ResetMethod resetMethod_;

    typedef void (IntelJpegAssistor::*InitWindowMethod)();
    InitWindowMethod initWindowMethod_;

    // Stuff for 32-bit mapping thru IPL
    unsigned char *ycbcr_buffer_;
    unsigned char *rgba_buffer_;

    IplImage *iplImage_;
    IplImage *workImage_;
    IplImage *sourceImage_;

    IplROI lockedROI_;
    IplImage *lockedImage_;

    IplColorTwist *twist_;
};

int __STDCALL errorCallback(IPLStatus status, const char * funcname,
		  const char * context, const char * file, int line)
{
    debug_msg("IPL error: %s: %s %s at %s:%d\n",
	iplErrorStr(status), funcname, context, file, line);
    return 0;
}

static class IntelJpegAssistorMatcher : public Matcher {
public:
    IntelJpegAssistorMatcher() : Matcher("assistor") 
    {
	iplRedirectError(errorCallback);
	debug_msg("create intel jpeg assistor\n");
    }

    TclObject* match(const char* id) 
    {
	debug_msg("match assistor to %s\n", id);
	if (strcmp(id, "jpeg/422") == 0) {
	    IntelJpegAssistor* p = new IntelJpegAssistor();
	    return (p);
	}
	return (0);
    }
} assistor_intel;

class WriteMeOnce
{
public:
    WriteMeOnce(void *data, int len, char *file)
    {
	FILE *fp = fopen(file, "wb");
	fwrite(data, len, 1, fp);
	fclose(fp);
    }
};

class WriteMeOnceRGB
{
public:
    WriteMeOnceRGB(void *data, int len, char *file)
    {
	FILE *fp = fopen(file, "wb");
	char *d = (char *) data;

	for (int i = 0; i < len; i += 4)
	{
	    fwrite(d + i, 3, 1, fp);
	}
	fclose(fp);
    }
};

IntelJpegAssistor::IntelJpegAssistor()
	: Renderer(FT_JPEG), imagesize_(0), window_(0),
	  method_(0), videoImage_(0), initWindowMethod_(0), resetMethod_(0),
	  ycbcr_buffer_(0), rgba_buffer_(0)
{
    debug_msg("create jpeg assistor\n");
}

IntelJpegAssistor::~IntelJpegAssistor()
{
    debug_msg("Destroy assistor\n");

    if (videoImage_ != 0)
	delete videoImage_;
}

int IntelJpegAssistor::command(int argc, const char*const* argv)
{
    if (argc == 3) 
    {
	if (strcmp(argv[1], "window") == 0) 
	{
	    window_ = VideoWindow::lookup(argv[2]);
	    
	    debug_msg("got window %s %x size=%dx%d\n", argv[2], window_,
		window_->width(), window_->height());
	    
	    if (window_->bpp() == 24)
	    {
		init_24();
	    }
	    else if (window_->bpp() == 32)
	    {
		init_32();
//		init_32_swapped();
	    }
	    
	    return (TCL_OK);
	}
    }
    return (Renderer::command(argc, argv));
}

int IntelJpegAssistor::consume(const VideoFrame* vf)
{
    if (method_ != 0)
    {
	int rc;
	
	JpegFrame *jf = (JpegFrame *) vf;
	
	if (jf->q_ != inq_ || jf->width_ != inw_ || jf->height_ != inh_)
	    reset(jf->q_, jf->width_, jf->height_);
	
	rc = (this->*method_)(jf);
	
	
	static int first = 1;
	if (first)
	{
	    FILE *fp = fopen("/temp/out.intel", "wb");
	    fwrite(prop_.DIBBytes, rw_ * rh_ * prop_.DIBChannels, 1, fp);
	    fclose(fp);
	    first = 0;
	}
	return rc;
	
    }
    else
	return 0;
}

int IntelJpegAssistor::consume_24_remap_direct(const JpegFrame * jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;
    
    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;
    
    
    err = ijlRead(&prop_, readType);
    
    
    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
	return 0;
    }

    videoImage_->checkRecalculate(Tk_WindowId(window_->tkwin()));

    DDrawVideoImage::LockedRegion lock;
    videoImage_->lockSurface(0, 0, 0, 0, rw_, rh_, lock);

    if (!lock.locked())
    {
	return 0;
    }

       
    lockedImage_->widthStep = lock.pitch();
    lockedImage_->width = lock.width();
    lockedImage_->height = lock.height();
    lockedImage_->imageData = (char *) lock.address();


    //
    // Do we need to resize?
    //

    if (rw_ < inw_ || rh_ < inh_)
    {
	iplDecimate(sourceImage_, lockedImage_, rw_, inw_, rh_, inh_, IPL_INTER_LINEAR);
    }
    else if (rw_ > inw_ || rh_ > inh_)
    {
	iplZoom(sourceImage_, lockedImage_, rw_, inw_, rh_, inh_, IPL_INTER_LINEAR);
    }

 
    return 0;
}

int IntelJpegAssistor::consume_24_remap(const JpegFrame * jf)
{
    // debug_msg("jpeg consume\n");
    
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;
    
    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;
    
    
    err = ijlRead(&prop_, readType);
    
    
    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
	return 0;
    }

    //
    // Do we need to resize?
    //

    if (rw_ < inw_ || rh_ < inh_)
    {
	iplDecimate(sourceImage_, iplImage_, rw_, inw_, rh_, inh_, IPL_INTER_LINEAR);
    }
    else if (rw_ > inw_ || rh_ > inh_)
    {
	iplZoom(sourceImage_, iplImage_, rw_, inw_, rh_, inh_, IPL_INTER_LINEAR);
    }

    window_->render(videoImage_, 0, rh_, 0, rw_);
    
    return 0;
}

int IntelJpegAssistor::consume_24_exact(const JpegFrame * jf)
{
    // debug_msg("jpeg consume\n");
    
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;
    
    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = IJL_YCBCR;
    
    
    err = ijlRead(&prop_, readType);
    
    static int first = 1;
    if (first)
    {
	FILE *fp = fopen("/temp/out.intel", "wb");
	fwrite(prop_.DIBBytes, rw_ * rh_ * prop_.DIBChannels, 1, fp);
	fclose(fp);
	first = 0;
    }
    
    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
    }
    else
    {
	window_->render(videoImage_, 0, rh_, 0, rw_);
    }
    
    return 0;
}


int IntelJpegAssistor::consume_32_direct(const JpegFrame* jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;
    
    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = (IJL_COLOR) IJL_YCBCR;
    
    err = ijlRead(&prop_, readType);
    
    
    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
	return 0;
    }

    IplImage *target;

    DDrawVideoImage::LockedRegion lock;
    videoImage_->lockSurface(0, 0, 0, 0, rw_, rh_, lock);

    if (!lock.locked())
    {
	return 0;
    }
    target = lockedImage_;
    
    
    target->widthStep = lock.pitch();
    target->width = lock.width();
    target->height = lock.height();
    target->imageData = (char *) lock.address();
    
    //
    // Do we need to resize?
    //
    
    if (rw_ < inw_ || rh_ < inh_)
    {
	iplDecimate(sourceImage_, target, rw_, inw_, rh_, inh_, IPL_INTER_NN);
	iplApplyColorTwist(target, target, twist_, 0);
    }
    else if (rw_ > inw_ || rh_ > inh_)
    {
	iplApplyColorTwist(sourceImage_, sourceImage_, twist_, 0);
	iplZoom(sourceImage_, target, rw_, inw_, rh_, inh_, IPL_INTER_NN);
    }
    else
    {
        iplApplyColorTwist(sourceImage_, target, twist_, 0);
    }

    videoImage_->checkRecalculate(Tk_WindowId(window_->tkwin()));

    return 0;
}


int IntelJpegAssistor::consume_32_via_render(const JpegFrame* jf)
{
    IJLERR err;
    IJLIOTYPE readType = IJL_JBUFF_READWHOLEIMAGE;
    
    prop_.JPGBytes = jf->bp_;
    prop_.JPGSizeBytes = jf->len_;
    prop_.JPGColor = (IJL_COLOR) IJL_YCBCR;
    
    err = ijlRead(&prop_, readType);
    
    if (err < 0)
    {
	debug_msg("ijlRead failed: %s\n", ijlErrorStr(err));
	return 0;
    }

    if (rw_ < inw_ || rh_ < inh_)
    {
	iplDecimate(sourceImage_, iplImage_, rw_, inw_, rh_, inh_, IPL_INTER_NN);
	iplApplyColorTwist(iplImage_, iplImage_, twist_, 0);
    }
    else if (rw_ > inw_ || rh_ > inh_)
    {
	iplApplyColorTwist(sourceImage_, sourceImage_, twist_, 0);
	iplZoom(sourceImage_, iplImage_, rw_, inw_, rh_, inh_, IPL_INTER_NN);
    }
    else
    {
        iplApplyColorTwist(sourceImage_, iplImage_, twist_, 0);
    }

    window_->render(videoImage_, 0, rh_, 0, rw_);

    return 0;
}

void IntelJpegAssistor::init_common()
{
    debug_msg("init jpeg:\n");
    debug_msg("  window size=%dx%d\n", window_->width(), window_->height());
    
    ZeroMemory( &prop_, sizeof( JPEG_CORE_PROPERTIES ) );
    
    prop_.JPGSubsampling = IJL_422;
    
    IJLERR err = ijlInit(&prop_);
    
    if (err < 0)
    {
	debug_msg("ijlInit failed: %s\n", ijlErrorStr(err));
    }
}

void IntelJpegAssistor::init_24()
{
    init_common();
    
    prop_.DIBChannels = 3;
    prop_.DIBColor = IJL_BGR;
    
    method_ = &IntelJpegAssistor::consume_24_exact;
    initWindowMethod_ = &IntelJpegAssistor::init_window_24;
    resetMethod_ = &IntelJpegAssistor::reset_24;
}

void IntelJpegAssistor::init_window_24()
{
    
    prop_.DIBBytes = videoImage_->pixbuf();
    prop_.DIBWidth = rw_;
    prop_.DIBHeight = rh_;

    iplSetROI(&lockedROI_, 0, 0, 0, rw_, rh_);
    lockedImage_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	rw_, rh_, &lockedROI_, 0, 0, 0);
    if (lockedImage_ == 0)
    {
	debug_msg("create locked image header failed\n");
	IPL_ERRCHK("init_window_32", "iplCreateImageHeader");
   }
 
}

void IntelJpegAssistor::reset_24()
{
    if (inw_ == rw_ && inh_ == rh_)
	method_ = &IntelJpegAssistor::consume_24_exact;
    else
    {
	sourceImage_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	    IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	    inw_, inh_, 0, 0, 0, 0);
	IPL_ERRCHK("reset_24", "iplCreateImageHeader");
	
	if (sourceImage_ == 0)
	{
	    debug_msg("create image header failed\n");
	}
	
	debug_msg("create, image size is %dx%d\n", rw_, rh_);
	iplAllocateImage(sourceImage_, 0, 0);
	IPL_ERRCHK("reset_24", "iplAllocateImage");
	
	prop_.DIBBytes = (unsigned char*) sourceImage_->imageData;
	prop_.DIBWidth = inw_;
	prop_.DIBHeight = inh_;

	method_ = &IntelJpegAssistor::consume_24_remap_direct;
//  	method_ = &IntelJpegAssistor::consume_24_remap;
	
	debug_msg("create, image size is %dx%d\n", rw_, rh_);
	iplImage_ = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", 
	    IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_4BYTES, 
	    rw_, rh_, 0, 0, 0, 0);
	
	IPL_ERRCHK("reset_24", "iplCreateImageHeader");
	
	if (iplImage_ == 0)
	{
	    debug_msg("create image header failed\n");
	}
	
	iplImage_->imageData = (char *) videoImage_->pixbuf();
	iplImage_->imageSize = rw_ * rh_ * 3;

   }
}


/*
 * JPEG assist for 32-bit displays must flow through
 * an IPL image in order to get the reversed-byteorder
 * pixels for direct draw.
 */
void IntelJpegAssistor::init_32()
{
    init_common();

#if 0
    prop_.DIBChannels = 3;
    prop_.DIBColor = (IJL_COLOR) IJL_OTHER;
#else
    prop_.DIBChannels = 4;
    prop_.DIBColor = IJL_RGBA_FPX;
#endif
				
    method_ = &IntelJpegAssistor::consume_32_via_render;
    initWindowMethod_ = &IntelJpegAssistor::init_window_32;
    resetMethod_ = &IntelJpegAssistor::reset_32;

    static int rgba_bgra_twist_data[] = { 0, 0, 1, 0, 
					0, 1, 0, 0,
					1, 0, 0, 0, 
					0, 0, 0, 1 };
    twist_ = iplCreateColorTwist(rgba_bgra_twist_data, 0);
}

void IntelJpegAssistor::init_32_swapped()
{
    init_common();
    
    prop_.DIBChannels = 4;
    prop_.DIBColor = IJL_RGBA_FPX;
				
    method_ = &IntelJpegAssistor::consume_32_via_render;
    initWindowMethod_ = &IntelJpegAssistor::init_window_32;
    resetMethod_ = &IntelJpegAssistor::reset_32;
}

void IntelJpegAssistor::init_window_32()
{
    //
    // Set up an IPL image header pointing to the video image's pixel 
    // buffer.
    //
    
    iplSetROI(&lockedROI_, 0, 0, 0, rw_, rh_);
    lockedImage_ = iplCreateImageHeader(4, 0, IPL_DEPTH_8U, "RGBA", "RGBA", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	rw_, rh_, &lockedROI_, 0, 0, 0);
    if (lockedImage_ == 0)
    {
	debug_msg("create locked image header failed\n");
	IPL_ERRCHK("init_window_32", "iplCreateImageHeader");
   }


    iplImage_ = iplCreateImageHeader(4, 0, IPL_DEPTH_8U, "RGBA", "RGBA", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	rw_, rh_, 0, 0, 0, 0);

    IPL_ERRCHK("init_window_32", "iplCreateImageHeader");

    if (iplImage_ == 0)
    {
	debug_msg("create image header failed\n");
    }


    iplImage_->imageData = (char *) videoImage_->pixbuf();
    iplImage_->imageSize = rw_ * rh_ * 4;

    workImage_ = iplCreateImageHeader(4, 0, IPL_DEPTH_8U, "RGBA", "RGBA", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	rw_, rh_, 0, 0, 0, 0);

    IPL_ERRCHK("init_window_32", "iplCreateImageHeader");

    if (workImage_ == 0)
    {
	debug_msg("create image header failed\n");
    }
    iplAllocateImage(workImage_, 0, 0);
}

void IntelJpegAssistor::reset_32()
{
    sourceImage_ = iplCreateImageHeader(4, 0, IPL_DEPTH_8U, "RGBA", "RGBA", 
	IPL_DATA_ORDER_PIXEL, IPL_ORIGIN_TL, IPL_ALIGN_8BYTES, 
	inw_, inh_, 0, 0, 0, 0);
    IPL_ERRCHK("reset_32", "iplCreateImageHeader");

    if (sourceImage_ == 0)
    {
	debug_msg("create image header failed\n");
    }

    iplAllocateImage(sourceImage_, 0, 0);
    IPL_ERRCHK("reset_32", "iplAllocateImage");

    prop_.DIBBytes = (unsigned char*) sourceImage_->imageData;
    prop_.DIBWidth = inw_;
    prop_.DIBHeight = inh_;

}

void IntelJpegAssistor::reset(int inq, int inw, int inh)
{
    
    debug_msg("jpeg reset q=%d w=%d h=%d\n", inq, inw, inh);
    /*
    * Make sure all outstanding putimage's finish.
    * Syncing with the first window, syncs the server,
    * and hence all of them.
    */
    window_->sync();
    
    Module::size(inw, inh);
    inw_ = inw;
    inh_ = inh;
    inq_ = inq;
    
    int w = window_->width();
    int h = window_->height();
    
    prop_.JPGWidth = inw_;
    prop_.JPGHeight = inh_;
    // prop_.jquality = inq_;

    
    if (w != rw_ || h != rh_) {
	rw_ = w;
	rh_ = h;
	
	if (videoImage_ != 0)
	    delete videoImage_;
	

	videoImage_ = new DDrawVideoImage(window_->tkwin(), rw_, rh_);
	
	(this->*initWindowMethod_)();
	
	//		allocimage();
    }
    (this->*resetMethod_)();
}