############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gramClient import GramClient, JOB_STATE_PENDING, JOB_STATE_ACTIVE
from pyGlobus.gramClient import JOB_STATE_FAILED, JOB_STATE_DONE, JOB_STATE_ALL, GramClientJobStateNames 
from pyGlobus.gramClient import GramClientAttr
from pyGlobus.security import GSSCred  
from pyGlobus.util import GlobusException
import sys, ConfigParser, unittest, time
from threading import * 

config = None # ConfigParser object


def func(cv, contact, state, error, error2='default'):
    #print 'begin func'
    str =  "**ENTERING PYTHON CALLBACK**    state="
    if state!=0: str += GramClientJobStateNames[state]

    #print str,' ',error
    #if error2 != 'default':
    #    print 'ERROR2 occured:',error2

    if state == JOB_STATE_PENDING: pass
    elif state == JOB_STATE_ACTIVE: pass
    elif state == JOB_STATE_FAILED:
        if cv:
            cv.acquire()  
            cv.notify()
            cv.release()
            #print '   [passed JOB_STATE_FAILED]'
    elif state == JOB_STATE_DONE:
        if cv:
            cv.acquire()
            cv.notify()
            cv.release()
        #print '   [passed JOB_STATE_DONE]'
    #print '**FINISHING PYTHON CALLBACK**'
    return


        
#############################################################################
#  Unit test for the GramClient Module:
#  Given a remote machine and an rsl command, the test function creates a
#  GramClient object and which sends the rsl message to the remote machine
#  and if all goes well then the command is executed.
#
##############################################################################


class GramClientTestCase(unittest.TestCase):
    "Test case for the GramClient module"
    
    def checkGramClient(self):  # tests {set_callback, submit_request}
        global config
        print  "\nGramClientTestCase begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl").strip('"')
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\nGramClientTestCase finished\n"
        except GlobusException, ex:
            assert 0, str(ex)

    def check_register_check_status(self):
        global config
        print  "\ncheck_register_check_status begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl").strip('"')

        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            gcAttr = GramClientAttr()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_check_status(jobContact, gcAttr, func, None)            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheck register_check_status finished\n"
        except GlobusException, ex:
            assert 0, str(ex)
                        
    def check_unregister_callback(self):
        global config
        print  "\ncheck_unregister_callback begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_callback(jobContact, 0, callbackContact) # 0 = state mask
            gramClient.unregister_callback(jobContact, callbackContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheck_unregister_callback finished\n"
        except GlobusException, ex:
            assert 0, str(ex)
            
    def check_register_callback_registration(self):
        global config
        print  "\nGramClientTestCase::check_register_callback_registration begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()            
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr()
            cred = GSSCred()
            gcAttr.set_credential(cred)
            assert not gcAttr == None, "Error creating the GramClientAttr object"            
            condV.acquire()
            t1 = time.time()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            
            status = gramClient.register_callback_registration(jobContact, 0, 
                                                               callbackContact, gcAttr, func, None)
            # status is coming back None, which is bad. 
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time()
            #print 'TIMEDELTA: ',t2-t1
            if t2-t1>20:
                assert 0, 'GramClientTestCase::check_callback_registration problem (timeout>20) '
            print 'GramClientTestCase::check_register_callback_registration end.'        
        except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)

   
    def check_register_callback(self): 
        global config
        print  "\ncheck_register_callback begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_callback(jobContact, 0, callbackContact) # 0 = state mask
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheck_register_callback finished\n"
        except GlobusException, ex:
            assert 0, str(ex)
            

    def check_check_status(self):
        global config
        print  "\ncheckCheckStatus begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            status, tcode = gramClient.check_status(jobContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheckCheckStatus finished\n"            
        except GlobusException, ex:
            assert 0, str(ex)


    def checkRefreshCredentials(self):
        global config
        print  "\ncheckRefreshCredentials begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)

            cred = GSSCred()
            jcontact = gramClient.refresh_credentials(jobContact,cred)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheckRefreshCredentials finished\n"            
        except GlobusException, ex:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected!'
                assert 0, str(ex)

    def checkVersion(self):
        """
        makes sure gramClient.version() is 2
        """
        global config
        print  "\nGramClientTestCase::checkVersion begin.\n"
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            assert str(gramClient.version()) == "2", "Wrong gramClient.version()! Expected '2'!"             
        except GlobusException, ex:
            assert 0, ex.args
        print 'Done with checkVersion'

    def checkCancel(self):
        global config
        print  "\nGramClientTestCase::checkCancel begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            t1 = time.time()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.cancel_job(jobContact)            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            if t1-time.time()>20:
                assert 0, 'GramClientTestCase::checkCancel problem (timeout>20) '
            print 'GramClientTestCase::checkCancel end.'        
        except GlobusException, ex:
            assert 0, ex.args
            

    def checkRegisterRefreshCredentials(self):
        global config
        print  "\nGramClientTestCase::checkRegisterRefreshCredentials begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr()

            condV.acquire()
            t1 = time.time()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            nocredconst = GSSCred(handle=None) ## NOBUG??
            gramClient.register_refresh_credentials(jobContact, nocredconst, gcAttr, func, None)
            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time()
            print 'TIMEDELTA: ',t2-t1

            print 'GramClientTestCase::checkRegisterRefreshCredentials end.'        
        except GlobusException, ex:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected!'
                assert 0, ex.args
            
    def checkCancelNonBlocking(self):
        """
        Tests register_cancel_job and submit_request
        """
        global config
        print  "\nGramClientTestCase::checkCancelNonBlocking begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()            
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr()
        except GlobusException, s:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected0!', s.args
                assert 0, s.args
            cred = GSSCred()
            gcAttr.set_credential(cred)
            assert not gcAttr == None, "Error creating the GramClientAttr object"            
            condV.acquire()
            t1 = time.time()
        except GlobusException, s:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected0!', s.args
                assert 0, s.args
            
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_cancel_job(jobContact, gcAttr, func, None)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time()
            #print 'TIMEDELTA: ',t2-t1
            if t2-t1>20:
                assert 0, 'GramClientTestCase::checkCancelNonBlocking problem (timeout>20) '
            print 'GramClientTestCase::checkCancelNonBlocking end.'        
        except GlobusException, s:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected1!', s.args
                assert 0, s.args
            

    def checkPing(self):
        global config
        print  "\nGramClient::CheckPing begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        rm = config.get("gramClient", "rm")
        print 'rm: ',rm 
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            x = gramClient.ping(rm)            
        except GlobusException, ex:
            print 'Failed checkPing in test_GramClient.py!'
            assert 0, ex.args
        print 'done with ping'

    def checkGramRegister_Sumbit_Request(self):
        global config
        print  "\ncheckGramRegister_Submit_Request begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl")
        rsl = rsl.strip('"')
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            gcAttr = GramClientAttr()
            cred = GSSCred()
            gcAttr.set_credential(cred)
            assert not gcAttr == None, "Error creating the GramClientAttr object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.register_submit_request(rm, rsl, 
                                                        JOB_STATE_ALL, callbackContact,
                                                        gcAttr, func, None)
           
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\ncheckGramRegister_Submit_Request finished\n"
        except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)


class GramClientAttrTestCase(unittest.TestCase):
    def TestGetSets(self):
	self.cred = GSSCred()
        self.cred.acquire_cred()
        print "GramClientAttrTestCase begin--------------------------"

	try:
	    gca  = GramClientAttr()
	    gca.set_credential(self.cred)  # tests set_credential()
	except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)

	try: gca.get_credential()  # tests get_credential()
	except GlobusException, s:
            if str(s)=="globus_gram_client_attr_get_credential is unimplemented in this version of gram_client":
                print 'expected'
            else: print str(s)

        try: print 'handle=',gca.get_handle()
        except GlobusException, s:
            if str(s)=="globus_gram_client_attr_get_credential is unimplemented in this version of gram_client":
                print 'expected'
            else: print str(s)
            
	print "GramClientAttrTestCase end--------------------------"


def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GramClientAttrTestCase,"Test")) 
    suite.addTest(unittest.makeSuite(GramClientTestCase, "check")) 
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

