############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import unittest,ConfigParser
from pyGlobus.xio import XioAttr
from pyGlobus.xio import XioStack
from pyGlobus.xio import XioDriver
from pyGlobus.xio import XioFile
from pyGlobus.xio import XioFileAsyn
from pyGlobus.xio import XioServer
from pyGlobus.xio import XioServerAsyn
from pyGlobus.xio import XioDataDescriptor
from pyGlobus import xioc
from pyGlobus.xio import XioException
from pyGlobus.util import Buffer
from threading import *

config = None # ConfigParser object
config_file = 'config.txt'


class TestClientAsynch(Thread):
    def print_error(self, errTuple):
        if errTuple[0] != 0:
            print errTuple[1]
        #else:
            #print "No errors"
   
    def openCb(self, condV, handle, errTuple):
        self.print_error(errTuple)
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()
        
    def writeCb(self, condV,handle, errTuple, buffer, nbytest, data_desc):
        self.print_error(errTuple)
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()

    def closeFileCb(self, condV,handle, errTuple):
        self.print_error(errTuple)
        condV.acquire()
        self._cond =1
        condV.notify()
        condV.release()

    def run(self):
        try:
            stack = XioStack()
            tcpDriver = XioDriver()
            tcpDriver.load("tcp")
            stack.push_driver(tcpDriver)
            file = XioFileAsyn(stack)
            condV = Condition(Lock())
            self._cond = 0
            condV.acquire()
            file.open(self.openCb, condV,g_contact)
            while self._cond == 0:
                condV.wait()
            condV.release()
            str = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
            
            self._cond = 0
            condV.acquire()
            file.write(self.writeCb, condV, str)
            while self._cond == 0:
                condV.wait()
            condV.release()
            self._cond = 0
            condV.acquire()
            file.close(self.closeFileCb, condV)
            while self._cond == 0:
                condV.wait()
            condV.release()
            tcpDriver.unload()
            del stack
        except XioException,ex:
            print ex


class TestServerAsynch:
    def print_error(self, errTuple):
        if errTuple[0] != 0:
            print errTuple[1]
        else:
            pass
            #print "No errors in callback"
    
    def acceptCb(self, condV, server, handle, errTuple):
        self.print_error(errTuple)
        self._file = XioFileAsyn(None,handle)
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()
    

    def openCb(self, condV, handle, errTuple):
        self.print_error(errTuple)
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()
        
    def readCb(self, condV,handle, errTuple, buffer, nbytes, data_desc):
        self.print_error(errTuple)
        self._buffer = buffer
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()

    def closeFileCb(self, condV,handle, errTuple):
        self.print_error(errTuple)
        condV.acquire()
        self._cond = 1
        condV.notify()
        condV.release()
        
    def closeCb(self,condV, server):
        condV.acquire()
        self._cond =1
        condV.notify()
        condV.release()

        
    def run(self):
        global g_contact
        try:
            stack = XioStack()
            tcpDriver = XioDriver()
            tcpDriver.load("tcp")
            stack.push_driver(tcpDriver)
            serverAttr= XioAttr()
            server = XioServerAsyn(serverAttr, stack)
            condV = Condition(Lock())
            
            g_contact = server.get_contact()
            client = TestClientAsynch()
            client.start()
            condV.acquire()
            self._cond = 0
            server.accept(self.acceptCb, condV )
            while self._cond == 0:
                condV.wait()
            condV.release()
            condV.acquire()
            self._cond = 0
            self._file.open(self.openCb, condV, g_contact)
            while self._cond == 0:
                condV.wait()
            condV.release()
            readBuffer = Buffer(46)
            condV.acquire()
            self._cond = 0
            self._file.read(self.readCb, condV, readBuffer)
            while self._cond == 0:
                condV.wait()
            condV.release()
            print "read from socket: ", self._buffer
            del readBuffer
            condV.acquire()
            self._cond = 0
            self._file.close(self.closeFileCb, condV)
            while self._cond == 0:
                condV.wait()
            condV.release()
            condV.acquire()
            self._cond = 0
            server.close(self.closeCb, condV)
            while self._cond == 0:
                condV.wait()
            condV.release()
            del self._file
            tcpDriver.unload()
            del tcpDriver
        except XioException, ex:
            print ex

class TestServer:

    def run(self):
        try:
            stackAttr = XioAttr()
            stack = XioStack(stackAttr)
            tcpDriver = XioDriver()
            tcpDriver.load("tcp")
            stack.push_driver(tcpDriver)
            serverAttr = XioAttr()
            port = config.get("xio", "ListenPort")
            serverAttr.cntl(tcpDriver, xioc.GLOBUS_XIO_TCP_SET_PORT, int(port))
            server = XioServer(serverAttr, stack)
            contact = server.get_contact()
            print "serving at ",contact
            client = TestClient()
            client.setPort(port)
            client.start()
            file = server.accept()
            fileAttr = XioAttr()
            file.open(contact,fileAttr)
            readBuffer = Buffer(46)
            outString = file.read(readBuffer, 8)
            print outString
            file.close(fileAttr)
            server.close()
          
        except XioException,ex:
            print ex
            raise ex

    def setPort(self, port):
        self._port = port

class TestClient(Thread):
    
    def run(self):
        try:
            print "running TestClient"
            attr = XioAttr()
            contact = "localhost:" + self._port
            stack = XioStack(attr)
            tcpDriver = XioDriver()
            tcpDriver.load("tcp")
            stack.push_driver(tcpDriver)
            file = XioFile(stack)
            fileAttr = XioAttr()
            file.open(contact,fileAttr)
            str = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
            file.write(str)
            file.close(fileAttr)
            tcpDriver.unload()
            del stack
        except XioException,ex:
            print "HERE in except"
            print ex

    def setPort(self, port):
        self._port = port

    
class XioServerTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
    def tearDown(self):
        print "DONE"

    def checkCreate(self):
        server = TestServer()
        server.run()
    

class XioServerAsyncTestCase(unittest.TestCase):
    def tearDown(self):
        print "DONE"
        
    def checkCreate(self):
        server = TestServerAsynch()
        server.run()
        

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(XioServerTestCase, "check"))
    suite.addTest(unittest.makeSuite(XioServerAsyncTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
